from glob import glob
from argparse import ArgumentParser
from os.path import join, basename, splitext
import subprocess
import time
import multiprocessing
import time


def execute_benchmarking(solver, encoding):
    with open("output_" + encoding + "_" + solver + ".txt", 'w') as out:
        for graphSize in range(2, 21):
            print("Encoding: " + encoding + "; Solver: " + solver + "; Graph size: " + str(graphSize))
            endtime = time.time() + TIMEOUT
            for filename in glob(join(path, '*.dzn')):
                remainingTime = endtime - time.time()
                if remainingTime < 0:
                    break
                graphName = splitext(basename(filename))[0]
                currGraphSize = int(graphName[:2])
                if graphSize == currGraphSize:
                    try:
                        out.write("\n")
                        out.write("##### " + filename + "\n")
                        out.flush()
                        if solver == "COIN-BC":
                            # no random seed
                            subprocess.run(
                                "minizinc --solver " + solver + " encodings\\" + encoding + ".mzn " + filename + " --no-intermediate --statistics --output-time",
                                stdout=out, text=True, timeout=remainingTime)
                        else:
                            subprocess.run(
                                "minizinc --solver " + solver + " encodings\\" + encoding + ".mzn " + filename + " --no-intermediate --statistics --output-time --random-seed 111730664",
                                stdout=out, text=True, timeout=remainingTime)
                    except subprocess.TimeoutExpired:
                        out.write("timeout after " + str(remainingTime) + "seconds\n")
                        out.flush()


path = '.\\input_data\\all_graphs_sorted\\'
TIMEOUT = 30 * 60   # running time per graph size

if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument("-s", "--solver", dest="solver", help="Specify solver")
    args = parser.parse_args()
    solver = args.solver
    print("Solver: " + solver + "\n")

    encodings = ["treewidth1", "treewidth2", "pathwidth1", "pathwidth2", "pathwidth3", "pathwidth4"]

    # execute every encoding in different process
    procs = []
    for encoding in encodings:
        p = multiprocessing.Process(target=execute_benchmarking, args=(solver, encoding, ), name=('process_' + encoding))
        procs.append(p)
        p.start()
        print('starting', p.name)

    for p in procs:
        p.join(5)
